@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ad-blocker.c,v 1.9.2.1 2006/04/09 20:24:23 chpe Exp $
+ *  $Id: ad-blocker.c,v 1.10 2006/05/17 20:04:56 jframeau Exp $
  */
 
 #include "config.h"
@@ -31,15 +31,12 @@ static void ad_blocker_init (AdBlocker *dialog);
 
 struct _AdBlockerPrivate
 {
-	AdUriTester *uri_tester;
-
 	int num_blocked;
 };
 
 enum
 {
 	PROP_0,
-	PROP_URI_TESTER,
 	PROP_NUM_BLOCKED
 };
 
@@ -86,9 +83,9 @@ ad_blocker_new (void)
 void
 ad_blocker_blocked_uri (AdBlocker *blocker)
 {
-	blocker->priv->num_blocked++;
+		blocker->priv->num_blocked++;
 
-	g_object_notify (G_OBJECT (blocker), "num-blocked");
+		g_object_notify (G_OBJECT (blocker), "num-blocked");
 }
 
 void
@@ -140,10 +137,6 @@ ad_blocker_init (AdBlocker *blocker)
 static void
 ad_blocker_finalize (GObject *object)
 {
-	/*
-	AdBlockerPrivate *priv = AD_BLOCKER_GET_PRIVATE (AD_BLOCKER (object));
-	*/
-
 	LOG ("AdBlocker finalizing %p", object);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
