@@ -15,17 +15,19 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: extensions-manager-ui.c,v 1.18 2006/02/07 23:08:55 crispin Exp $
+ *  $Id: extensions-manager-ui.c,v 1.21 2006/04/09 20:23:02 chpe Exp $
  */
 
 #include "config.h"
 
 #include "extensions-manager-ui.h"
-#include "ephy-debug.h"
 
 #include <epiphany/ephy-extensions-manager.h>
 #include <epiphany/ephy-shell.h>
 
+#include "ephy-gui.h"
+#include "ephy-debug.h"
+
 #include <gtk/gtkaboutdialog.h>
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtkcellrenderertoggle.h>
@@ -38,6 +40,7 @@
 #include <string.h>
 
 #include <glib/gi18n-lib.h>
+#include <gtk/gtkversion.h>
 
 #define IGNORE_SELF /* Don't display *this* extension in the list */
 #define GROUP	"Epiphany Extension"
@@ -123,7 +126,10 @@ extensions_manager_ui_response_cb (GtkWidget *widget,
 {
 	if (response == GTK_RESPONSE_HELP)
 	{
-		/* FIXME not implemented yet */
+		ephy_gui_help_with_doc_id (GTK_WINDOW (widget),
+					   "epiphany-extensions",
+					   "epiphany-extensions",
+					   "epi-ext-manager");
 		return;
 	}
 
@@ -151,7 +157,16 @@ show_extension_info (ExtensionsManagerUI *parent_dialog,
 
 	gtk_window_set_transient_for (GTK_WINDOW (dialog),
 				      GTK_WINDOW (parent_dialog->priv->window));
-	/* FIXME window group ! */
+#if !GTK_CHECK_VERSION(2,9,0)
+	gtk_window_group_add_window (ephy_gui_ensure_window_group (GTK_WINDOW (parent_dialog->priv->window)),
+				     GTK_WINDOW (dialog));
+#endif
+	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+
+#if GTK_CHECK_VERSION(2,9,0)
+	g_signal_connect (dialog, "response", G_CALLBACK (gtk_widget_destroy), NULL);
+#endif
+
 	gtk_window_present (GTK_WINDOW (dialog));
 
 	g_free (name);
