@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: extension.c,v 1.3 2005/01/13 16:15:49 chpe Exp $
+ *  $Id: extension.c,v 1.4 2005/01/28 22:30:25 chpe Exp $
  */
 
 #include "config.h"
@@ -32,7 +32,7 @@ G_MODULE_EXPORT GType register_module (GTypeModule *module);
 G_MODULE_EXPORT GType
 register_module (GTypeModule *module)
 {
-	LOG ("Registering EphySelectStylesheetExtension")
+	LOG ("Registering EphySelectStylesheetExtension");
 
 #ifdef ENABLE_NLS
 	/* Initialize the i18n stuff */
